/*-----------------------------------------------------------------------*/
/*                                   TIGA                                */
/*         Copyright (C) 1988-1991  Texas Instruments Incorporated.      */
/*                           All Rights Reserved                         */
/*-----------------------------------------------------------------------*/
#define TRUE    1
#define FALSE   0
#define NULL    0
#define DWORD   unsigned long
#define WORD    unsigned short
#define BYTE    unsigned char

/*----------------------------------------------------------------------*/
/* Define initial memory structure                                      */
/*----------------------------------------------------------------------*/
struct linMem
{
    char          *start_address;
    unsigned long length;
};

typedef struct
{
    short         handles;
    short         grow_size;
    unsigned long stack_size;
    short         segments;
    struct linMem segs[1];
} MINIT_STRUCT;

#define LMEM_DEFINE(n) struct { short         handles;           \
                                short         grow_size;         \
                                unsigned long stack_size;        \
                                short         segments;          \
                                struct linMem segs[n]; }

/*----------------------------------------------------------------------*/
/* This structure is the header of a memory block.  It is included in   */
/* every free and allocated block.                                      */
/*----------------------------------------------------------------------*/
typedef struct Block
{
    BYTE             checkByte;         /* Header validation      */
    BYTE             flags;             /* Flags (see below)      */
    BYTE             segment;           /* Parent Segment of this block  */
    BYTE             dummy[3];
    struct Block     *prev;             /* Pointer-previous block */
    unsigned long    size;              /* size of block          */
    unsigned short   handle;            /* Block master pointer   */
    void             (*func)();
} BLOCKHDR;

/*----------------------------------------------------------------------*/
/* These are the definitions of each of the flags in the flags byte     */
/* of the memory block header.                                          */
/*----------------------------------------------------------------------*/
#define BLK_DELETABLE   0x01    /* Block can be deleted */
#define BLK_LOCKED      0x02    /* Block is locked */
#define BLK_FUNCMOVE    0x04    /* Call block function on move */
#define BLK_FUNCDELETE  0x08    /* Call block function on purge */
#define BLK_SECURED     0x10    /* Block is secured memory */
#define BLK_INUSE       0x20    /* Block is allocated */
#define BLK_LAST        0x80    /* Last block in segment */
#define BLK_USERFLAGS   0x1F    /* Group of User Setable Flags */

/*----------------------------------------------------------------------*/
/* freePtr is stored at the beginning of the user part of a             */
/* free block.  It contains the links to next and previous free blocks  */
/* in the segment.  prev is NULL if this is the first free block, and   */
/* next is NULL if this is the last block                               */
/*----------------------------------------------------------------------*/
typedef struct
{
    BLOCKHDR  *prev;   /* Pointer to previous free block */
    BLOCKHDR  *next;   /* Pointer to next free block */
} FREEBLK;

/*----------------------------------------------------------------------*/
/* Macro to return address of data area give block header address       */
/*----------------------------------------------------------------------*/
#define DATALOC(xx)     ((BYTE *) (xx) + sizeof(BLOCKHDR))

/*----------------------------------------------------------------------*/
/* Macro to return address of header given data area address            */
/*----------------------------------------------------------------------*/
#define HEADERLOC(xx)   ((BYTE *) (xx) - sizeof(BLOCKHDR))

/*----------------------------------------------------------------------*/
/* There is an LMEM entry for every linear block or RAM declared.       */
/*----------------------------------------------------------------------*/
typedef struct
{
    void        *block;         /* Address, associated block */
    DWORD       size;           /* Size of block             */
    DWORD       freeSpace;      /* Amount of free space      */
    BLOCKHDR    *free;          /* First free block          */
    BLOCKHDR    *next;          /* Next reference free block */
} LMEM;

#define CB                      'T'

/*----------------------------------------------------------------------*/
/* MINREMAINDER is the smallest free block that can remain after        */
/* allocation.  Making this larger reduces fragmentation but wastes     */
/* more space.                                                          */
/*                                                                      */
/* MINBLOCKDATA is the smallest block data area that can be created.    */
/* This must be at least sizeof(FREEBLK) bytes so there is space for    */
/* the next and prev pointers stored in a free block.                   */
/*----------------------------------------------------------------------*/
#define MINREMAINDER    (sizeof(BLOCKHDR) + sizeof(FREEBLK))
#define MINBLOCKDATA    MINREMAINDER

short          gsph_minit();
unsigned short gsph_alloc();
unsigned short gsph_falloc();
void           gsph_free();
void           *gsph_deref();
void           gsph_memtype();
unsigned long  gsph_totalfree();
unsigned long  gsph_maxheap();
void           gsph_compact();
unsigned long  gsph_realloc();

short          gsph_sinit();
short          gsph_findmem();
